==> states_rebuilder-4.3.0.tar.gz: lib/src/basics/reactive_model.dart <==
part of '../rm.dart';

///A lightweight version of InjectedImp
class ReactiveModel<T> extends InjectedBase<T> {
  final Function() creator;
  factory ReactiveModel.create(T state) {
    return ReactiveModel(creator: () => state, initialState: state);
  }

  factory ReactiveModel.future(
    Future<T> Function() creator, {
    T? initialState,
  }) {
    return ReactiveModel(creator: creator, initialState: initialState);
  }
  factory ReactiveModel.stream(
    Stream<T> Function() creator, {
    T? initialState,
  }) {
    return ReactiveModel(creator: creator, initialState: initialState);
  }
  ReactiveModel({
    required this.creator,
    T? initialState,
    bool autoDisposeWhenNotUsed = true,
  }) {
    _reactiveModelState = ReactiveModelBase<T>(
      creator: creator,
      initialState: initialState,
      autoDisposeWhenNotUsed: autoDisposeWhenNotUsed,
      initializer: () {
        if (_reactiveModelState._isInitialized) {
          return;
        }
        _reactiveModelState
          .._isInitialized = true
          .._isDisposed = false
          .._snapState = SnapState._nothing(
            _reactiveModelState._initialState,
            kInitMessage,
            _reactiveModelState.debugPrintWhenNotifiedPreMessage,
          );

        _reactiveModelState._setInitialStateCreator(
          middleCreator: (crt) {
            return crt();
          },
          middleState: (snap) {
            if (snap.isWaiting) {
              if (snapState._infoMessage == kRefreshMessage) {
                return snap._copyWith(data: _reactiveModelState._initialState);
              } else {
                _reactiveModelState._snapState = snap;
                return null;
              }
            }

            // _reactiveModelState._initialState ??= snap.data;

            if (snapState._infoMessage == kInitMessage) {
              snap = snap._copyToIsIdle();
              _reactiveModelState._snapState = snap;
              return null; //Return null so do not rebuild
            }
            return snapState._infoMessage == kRefreshMessage
                ? snap._copyToIsIdle(
                    /*data: _reactiveModelState._initialState*/)
                : snap;
          },
          onDone: (snap) {
            return snap;
          },
        );
        _reactiveModelState._initialStateCreator!();
      },
    );
    _reactiveModelState.initializer();
  }
  set state(T s) {
    setState((_) => s);
  }

  T? get initialState => _reactiveModelState._initialState;
  ReactiveModelBase<T> get reactiveModelState => _reactiveModelState;
  SnapState<T>? middleSnap(SnapState<T> snap) {}

  @override
  SnapState<T>? _middleSnap(
    SnapState<T> snap, {
    On<void>? onSetState,
    void Function(T data)? onData,
    void Function(dynamic error)? onError,
  }) {
    snap = middleSnap(snap) ?? snap;
    if (snap.isWaiting) {
      if (snapState.isWaiting) {
        return null;
      }
      onSetState?.call(snap);
      return snap;
    }
    if (snap.hasError) {
      if (snap.error == snapState.error) {
        return null;
      }
      onSetState?.call(snap);
      onError?.call(snap.error);
      return snap;
    }

    if (snap.hasData) {
      if (snap._isImmutable == true && snap == snapState) {
        return null;
      }
      onSetState?.call(snap);
      onData?.call(snap.data!);
    }
    return snap;
  }

  // @override
  // Future<F> Function() future<F>(Future<F> Function(T s) future) {
  //   return () async {
  //     late F data;
  //     await future(state).then((d) {
  //       if (d is T) {
  //         snapState = snapState.copyToHasData(d);
  //       }
  //       data = d;
  //     }).catchError(
  //       (e, StackTrace s) {
  //         snapState = snapState._copyToHasError(
  //           e,
  //           () => this.future(future),
  //           stackTrace: s,
  //         );

  //         throw e;
  //       },
  //     );
  //     return data;
  //   };
  // }

  ConnectionState get connectionState =>
      _reactiveModelState._snapState._connectionState;

  VoidCallback observeForRebuild(void Function(ReactiveModel<T>? rm) fn) {
    return _reactiveModelState.listeners.addListenerForRebuild((_) => fn(this));
  }

  void addCleaner(VoidCallback fn) {
    _reactiveModelState.listeners.addCleaner(fn);
  }

  ///Exhaustively switch over all the possible statuses of [connectionState].
  ///Used mostly to return [Widget]s.
  R whenConnectionState<R>({
    required R Function() onIdle,
    required R Function() onWaiting,
    required R Function(T snapState) onData,
    required R Function(dynamic error) onError,
    bool catchError = true,
  }) {
    if (isIdle) {
      return onIdle.call();
    }
    if (hasError) {
      return onError.call(error);
    }
    if (isWaiting) {
      return onWaiting.call();
    }
    return onData.call(state);
  }

  int get observerLength => _reactiveModelState.listeners.observerLength;
}

==> states_rebuilder-4.4.0-dev.tar.gz: lib/src/basics/reactive_model.dart <==
part of '../rm.dart';

///A lightweight version of InjectedImp
class ReactiveModel<T> extends InjectedBase<T> {
  final Function() creator;
  factory ReactiveModel.create(T state) {
    return ReactiveModel(creator: () => state, initialState: state);
  }

  factory ReactiveModel.future(
    Future<T> Function() creator, {
    T? initialState,
  }) {
    return ReactiveModel(creator: creator, initialState: initialState);
  }
  factory ReactiveModel.stream(
    Stream<T> Function() creator, {
    T? initialState,
  }) {
    return ReactiveModel(creator: creator, initialState: initialState);
  }
  ReactiveModel({
    required this.creator,
    T? initialState,
    bool autoDisposeWhenNotUsed = true,
  }) {
    _reactiveModelState = ReactiveModelBase<T>(
      creator: creator,
      initialState: initialState,
      autoDisposeWhenNotUsed: autoDisposeWhenNotUsed,
      initializer: () {
        if (_reactiveModelState._isInitialized) {
          return;
        }
        _reactiveModelState
          .._isInitialized = true
          .._isDisposed = false
          .._snapState = SnapState._nothing(
            _reactiveModelState._initialState,
            kInitMessage,
            _reactiveModelState.debugPrintWhenNotifiedPreMessage,
          );

        _reactiveModelState._setInitialStateCreator(
          middleCreator: (crt) {
            return crt();
          },
          middleState: (snap) {
            if (snap.isWaiting) {
              if (snapState._infoMessage == kRefreshMessage) {
                return snap._copyWith(data: _reactiveModelState._initialState);
              } else {
                _reactiveModelState._snapState = snap;
                return null;
              }
            }

            // _reactiveModelState._initialState ??= snap.data;

            if (snapState._infoMessage == kInitMessage) {
              snap = snap._copyToIsIdle();
              _reactiveModelState._snapState = snap;
              return null; //Return null so do not rebuild
            }
            return snapState._infoMessage == kRefreshMessage
                ? snap._copyToIsIdle(
                    /*data: _reactiveModelState._initialState*/)
                : snap;
          },
          onDone: (snap) {
            return snap;
          },
        );
        _reactiveModelState._initialStateCreator!();
      },
    );
    _reactiveModelState.initializer();
  }
  set state(T s) {
    setState((_) => s);
  }

  T? get initialState => _reactiveModelState._initialState;
  ReactiveModelBase<T> get reactiveModelState => _reactiveModelState;
  SnapState<T>? middleSnap(SnapState<T> snap) {}

  @override
  SnapState<T>? _middleSnap(
    SnapState<T> snap, {
    On<void>? onSetState,
    void Function(T data)? onData,
    void Function(dynamic error)? onError,
  }) {
    snap = middleSnap(snap) ?? snap;
    if (snap.isWaiting) {
      if (snapState.isWaiting) {
        return null;
      }
      onSetState?.call(snap);
      return snap;
    }
    if (snap.hasError) {
      if (snap.error == snapState.error) {
        return null;
      }
      onSetState?.call(snap);
      onError?.call(snap.error);
      return snap;
    }

    if (snap.hasData) {
      if (snap._isImmutable == true && snap == snapState) {
        return null;
      }
      onSetState?.call(snap);
      onData?.call(snap.data as T);
    }
    return snap;
  }

  // @override
  // Future<F> Function() future<F>(Future<F> Function(T s) future) {
  //   return () async {
  //     late F data;
  //     await future(state).then((d) {
  //       if (d is T) {
  //         snapState = snapState.copyToHasData(d);
  //       }
  //       data = d;
  //     }).catchError(
  //       (e, StackTrace s) {
  //         snapState = snapState._copyToHasError(
  //           e,
  //           () => this.future(future),
  //           stackTrace: s,
  //         );

  //         throw e;
  //       },
  //     );
  //     return data;
  //   };
  // }

  ConnectionState get connectionState =>
      _reactiveModelState._snapState._connectionState;

  VoidCallback observeForRebuild(void Function(ReactiveModel<T>? rm) fn) {
    return _reactiveModelState.listeners.addListenerForRebuild((_) => fn(this));
  }

  void addCleaner(VoidCallback fn) {
    _reactiveModelState.listeners.addCleaner(fn);
  }

  ///Exhaustively switch over all the possible statuses of [connectionState].
  ///Used mostly to return [Widget]s.
  R whenConnectionState<R>({
    required R Function() onIdle,
    required R Function() onWaiting,
    required R Function(T snapState) onData,
    required R Function(dynamic error) onError,
    bool catchError = true,
  }) {
    if (isIdle) {
      return onIdle.call();
    }
    if (hasError) {
      return onError.call(error);
    }
    if (isWaiting) {
      return onWaiting.call();
    }
    return onData.call(state);
  }

  int get observerLength => _reactiveModelState.listeners.observerLength;
}
